package com.mt.wms.basic.controller;

import com.mt.wms.basic.params.KilnInfoParam;
import com.mt.wms.basic.params.KilnInfoQueryParam;
import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.KilnInfo;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.validator.groups.AddGroup;
import com.mt.wms.core.validator.groups.PageGroup;
import com.mt.wms.core.validator.groups.UpdateGroup;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.groups.Default;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/9 12:56
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "kilnInfo")
@Slf4j
@Api(value = "工业炉基础信息管理", tags = "工业炉基础信息管理", hidden = false)
public class KilnInfoController extends BaseController {

    @Autowired
    private KilnInfoService kilnInfoService;

    @PostMapping(value = "get")
    @ApiOperation(value = "获取工业炉信息")
    private R<KilnInfoVo> get(@Validated @RequestBody IdParam idParam) {
        return kilnInfoService.get(idParam);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取工业炉列表")
    private R<List<KilnInfoVo>> list(@Validated({Default.class}) @RequestBody KilnInfoQueryParam kilnInfoQueryParam) {
        return kilnInfoService.list(kilnInfoQueryParam);
    }

    @PostMapping(value = "page")
    @ApiOperation(value = "获取分页工业炉")
    private R<PageVo<KilnInfoVo>> page(@Validated({PageGroup.class, Default.class}) @RequestBody KilnInfoQueryParam kilnInfoQueryParam) {
        return kilnInfoService.page(kilnInfoQueryParam);
    }

    @PostMapping(value = "add")
    @ApiOperation(value = "新增")
    private R<IdVo> add(@Validated({AddGroup.class, Default.class}) @RequestBody KilnInfoParam kilnInfoParam) {
        return kilnInfoService.add(kilnInfoParam);
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    private R<IdVo> update(@Validated({UpdateGroup.class, Default.class}) @RequestBody KilnInfoParam kilnInfoParam) {
        return kilnInfoService.update(kilnInfoParam);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    private R<IdVo> delete(@Validated @RequestBody IdParam idParam) {
        return kilnInfoService.delete(idParam);
    }

    @PostMapping(value = "codeGenerator")
    @ApiOperation(value = "编码生成")
    private R<String> codeGenerator(){
        return successful(IDGenerator.gen("YL", "yyyyMMddHHmm", 2, "KILN_CODE"));
    }
}
