package com.mt.wms.basic.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.basic.params.VehicleParam;
import com.mt.wms.basic.params.VehicleQueryParam;
import com.mt.wms.basic.service.VehicleService;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.VehicleInfo;
import com.mt.wms.core.dal.service.VehicleInfoServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.validator.groups.AddGroup;
import com.mt.wms.core.validator.groups.PageGroup;
import com.mt.wms.core.validator.groups.UpdateGroup;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.groups.Default;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/9 13:13
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "vehicle")
@Slf4j
@Api(value = "车辆基础信息管理", tags = "车辆基础信息管理", hidden = false)
public class VehicleController extends BaseController {

    @Autowired
    private VehicleService vehicleService;
    @Autowired
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @PostMapping(value = "get")
    @ApiOperation(value = "获取车辆信息")
    private R<VehicleVo> get(@Validated @RequestBody IdParam idParam) {
        return vehicleService.get(idParam);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取车辆列表")
    private R<List<VehicleVo>> list(@Validated({Default.class}) @RequestBody VehicleQueryParam vehicleQueryParam) {
        return vehicleService.list(vehicleQueryParam);
    }

    @PostMapping(value = "page")
    @ApiOperation(value = "获取分页车辆")
    private R<PageVo<VehicleVo>> page(@Validated({PageGroup.class, Default.class}) @RequestBody VehicleQueryParam vehicleQueryParam) {
        return vehicleService.page(vehicleQueryParam);
    }

    @PostMapping(value = "add")
    @ApiOperation(value = "新增")
    private R<IdVo> add(@Validated({AddGroup.class, Default.class}) @RequestBody VehicleParam vehicleParam) {
        return vehicleService.add(vehicleParam);
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    private R<IdVo> update(@Validated({UpdateGroup.class, Default.class}) @RequestBody VehicleParam vehicleParam) {
        return vehicleService.update(vehicleParam);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除")
    private R<IdVo> delete(@Validated @RequestBody IdParam idParam) {
        return vehicleService.delete(idParam);
    }

    @PostMapping(value = "codeGenerator")
    @ApiOperation(value = "编码生成")
    private R<String> codeGenerator(){
        return successful(IDGenerator.gen("CL", "yyyyMMddHHmm", 2, "VEHICLE_CODE"));
    }

    @PostMapping(value = "resetStatus")
    @ApiOperation(value = "重置车辆状态")
    private R<String> reset(){
        List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>().eq(VehicleInfo.STATUS, 1));
        for (VehicleInfo vehicleInfo:vehicleInfoList
             ) {
            vehicleInfo.setStatus(0);
            vehicleInfoServiceBiz.updateById(vehicleInfo);
        }
        return successful("操作成功，车辆状态已重置为空闲！");
    }
}
