package com.mt.wms.empty.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.JsonObject;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.ElectricQueryParam;
import com.mt.wms.empty.vo.ConsumeDataVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.validation.groups.Default;
import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

/**
 * @Author: liguanghao
 * @Date: 2021/12/16 11:20
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "electric")
@Slf4j
@Api(value = "电能消耗相关接口", tags = "电能消耗相关接口", hidden = false)
public class EnergyController extends BaseController {


    //@PostConstruct
    public void test(){
        LocalDateTime time=LocalDateTime.of(2021, 3, 31, 0, 0, 0);
        Float f=29000F;

        LocalDate timeDate=time.toLocalDate();
        LocalDateTime timeEnd = LocalDateTime.of(timeDate, LocalTime.MAX).minus(5,ChronoUnit.SECONDS);
        for (int i = 0; i < 10; i++) {

            ParEleValue parEleValue=new ParEleValue();
            parEleValue.setDateType(2);
            parEleValue.setParId(1);
            parEleValue.setKilnId(1L);
            parEleValue.setTotalBat(Float.valueOf(String.valueOf(f+(Math.random()*24000))));
            parEleValue.setCreateTime(timeEnd);
            timeEnd=timeEnd.plus(1,ChronoUnit.MONTHS);
            f=parEleValue.getTotalBat();
            parEleValueServiceBiz.save(parEleValue);
            System.out.println(parEleValue);
        }

    }

    @Autowired
    private ParEleValueServiceBiz parEleValueServiceBiz;
    @Autowired
    private ParGasValueServiceBiz parGasValueServiceBiz;
    @Autowired
    private ParRotSpeedValueServiceBiz parRotSpeedValueServiceBiz;
    @Autowired
    private ParTemValueServiceBiz parTemValueServiceBiz;
    @Autowired
    private KilnInfoServiceBiz kilnInfoServiceBiz;
    @Autowired
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Autowired
    private TaskHisServiceBiz taskHisServiceBiz;

    @PostMapping(value = "eletric")
    @ApiOperation(value = "根据工业炉id获取消耗情况")
    public R<String> list(@Validated({Default.class}) @RequestBody ElectricQueryParam electricQueryParam) {
        Long kiln = electricQueryParam.getKilnId();
        //工业炉id为0代表全部，存储的时候计算全部
            JSONObject jsonObject=new JSONObject();
            //指定时间节点
            LocalDate timeDate = electricQueryParam.getTime().toLocalDate();
            LocalDateTime timeStart = LocalDateTime.of(timeDate, LocalTime.MIN);
            LocalDateTime timeEnd = LocalDateTime.of(timeDate, LocalTime.MAX);
            //当前时间节点
            LocalDateTime todayStart = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
            LocalDateTime todayEnd = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);

            //总电能消耗
            List<ParEleValue> todayEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq("kiln_id", kiln)
                    .between("create_time", todayStart, todayEnd)
                    .orderByDesc("create_time"));
            Float countEleValue = todayEleValueList.get(0).getTotalBat();
            jsonObject.put("countEle",countEleValue);

            //昨日电能消耗
            LocalDateTime yesterdayStart = LocalDateTime.of(LocalDate.now().plus(1, ChronoUnit.DAYS), LocalTime.MIN);
            LocalDateTime yesterdayEnd = LocalDateTime.of(LocalDate.now().plus(1, ChronoUnit.DAYS), LocalTime.MAX);
            List<ParEleValue> yesterdayEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq("kiln_id", kiln)
                    .between("create_time", yesterdayStart, yesterdayEnd)
                    .eq("date_type",0)
                    .orderByAsc("create_time"));
            Float yesterdayTotalBat= yesterdayEleValueList.get(yesterdayEleValueList.size()-1).getTotalBat()-yesterdayEleValueList.get(0).getTotalBat();
            jsonObject.put("yesterdayCountEle",yesterdayTotalBat);

            //指定日期当天的电能消耗
            if (electricQueryParam.getDateType()==0){
                List<ParEleValue> dateEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", timeStart, timeEnd)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (dateEleValueList.size()!=0){
                    dateCountEle= dateEleValueList.get(dateEleValueList.size()-1).getTotalBat()-dateEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                //分时消耗量
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (int i = 0; i < dateEleValueList.size() - 1; ++i) {
                    // TODO: 2021/12/16
                    float eleValue = dateEleValueList.get(i).getTotalBat() - dateEleValueList.get(i - 1).getTotalBat();
                    int j=i+1;
                    if (j%2==0){
                        String time=(j-1)/2+":30";
                        timeJsonObject.put(time,eleValue);
                    }else {
                        String time=(j-1)/2+":00";
                        timeJsonObject.put(time,eleValue);
                    }
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当周的每天电能消耗
            if(electricQueryParam.getDateType()==1){
                LocalDate monday = timeDate.with(DayOfWeek.MONDAY);
                LocalDate sunday = timeDate.with(DayOfWeek.SUNDAY);
                List<ParEleValue> timeWeekEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", monday, sunday.plusDays(1))
                        .eq("date_type", 1)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeWeekEleValueList.size()!=0){
                    dateCountEle= timeWeekEleValueList.get(timeWeekEleValueList.size()-1).getTotalBat()-timeWeekEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeWeekEleValueList
                     ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当月的每天电能消耗
            if(electricQueryParam.getDateType()==2){
                LocalDate firstday = LocalDate.of(timeDate.getYear(), timeDate.getMonthValue(), 1);
                LocalDate lastDay = timeDate.with(TemporalAdjusters.lastDayOfMonth());
                List<ParEleValue> timeMonthEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", firstday, lastDay.plusDays(1))
                        .eq("date_type", 1)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeMonthEleValueList.size()!=0){
                    dateCountEle= timeMonthEleValueList.get(timeMonthEleValueList.size()-1).getTotalBat()-timeMonthEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeMonthEleValueList
                ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当年的每月电能消耗
            if(electricQueryParam.getDateType()==2){
                LocalDate firstDay = timeDate.with(TemporalAdjusters.firstDayOfYear());
                LocalDate lastDay = timeDate.with(TemporalAdjusters.lastDayOfYear());
                List<ParEleValue> timeYearEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", firstDay, lastDay.plusDays(1))
                        .eq("date_type", 2)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeYearEleValueList.size()!=0){
                    dateCountEle= timeYearEleValueList.get(timeYearEleValueList.size()-1).getTotalBat()-timeYearEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeYearEleValueList
                ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
        return successful(jsonObject.toJSONString());
    }
    @PostMapping(value = "energyList")
    @ApiOperation(value = "根据加工炉id获取能源消耗情况")
    public R<String> energyList(@Validated({Default.class}) @RequestBody ElectricQueryParam electricQueryParam){
        TaskHis currTask = taskHisServiceBiz.getById(electricQueryParam.getTaskId());
        Long kilnId = currTask.getKilnId();
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
        Integer kilnType = kilnInfo.getType();
        //传入任务号，工业炉号，查询消耗数据，返回json字符串
        JSONObject jsonObject=new JSONObject();
        ConsumeDataVo consumeDataVo=ConsumeDataVo.builder().build();

        Map<String,Object> map=new HashMap<>();
        //电能消耗
        List<ParEleValue> parEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                .eq(ParEleValue.KILN_ID, kilnId)
                .eq(ParEleValue.TASK_ID, electricQueryParam.getTaskId()));
        if (parEleValueList.size()>0){
            List<Object> eleList=new ArrayList<>();
            //电能消耗
            List<Object> eleConsume=new ArrayList<>();
            //A向电压
            List<Object> aVolConsume=new ArrayList<>();
            //B向电压
            List<Object> bVolConsume=new ArrayList<>();
            //C向电压
            List<Object> cVolConsume=new ArrayList<>();
            Float talBat=0F;
            int i=0;
            for (ParEleValue parEleValue:parEleValueList
                 ) {
                //首位置为0
                if (i==0){
                    talBat=parEleValue.getTotalBat();
                }

                i++;
                map.put("time",parEleValue.getCreateTime());
                map.put("value",floatSubtract(parEleValue.getTotalBat(),talBat));
                map.put("name","耗电量");
                map.put("unit","KW");
                eleConsume.add(JSONObject.toJSON(map));
                map.clear();
                map.put("time",parEleValue.getCreateTime());
                map.put("value",parEleValue.getAVoltagevValue());
                map.put("name","A向电压");
                map.put("unit","V");
                aVolConsume.add(JSONObject.toJSON(map));
                map.clear();
                map.put("time",parEleValue.getCreateTime());
                map.put("value",parEleValue.getBVoltagevValue());
                map.put("name","B向电压");
                map.put("unit","V");
                bVolConsume.add(JSONObject.toJSON(map));
                map.clear();
                map.put("time",parEleValue.getCreateTime());
                map.put("value",parEleValue.getCVoltagevValue());
                map.put("name","C向电压");
                map.put("unit","V");
                cVolConsume.add(JSONObject.toJSON(map));
                map.clear();
            }
            eleList.add(eleConsume);
            eleList.add(aVolConsume);
            eleList.add(bVolConsume);
            eleList.add(cVolConsume);
            jsonObject.put("ele",eleList);
        }
        //氮气、甲醇、丙烷、氨气消耗量
        List<ParGasValue> parGasValueList = parGasValueServiceBiz.list(new QueryWrapper<ParGasValue>()
                .eq(ParEleValue.KILN_ID, kilnId)
                .eq(ParEleValue.TASK_ID, electricQueryParam.getTaskId()));
        if (parGasValueList.size()>0){
            if (kilnType==1){
                List<Object> gasList=new ArrayList<>();
                //氮气
                List<Object> nitConsume=new ArrayList<>();
                Float nit=0F;
                //甲醇
                List<Object> metConsume=new ArrayList<>();
                Float met=0F;
                //丙烷
                List<Object> propaneConsume=new ArrayList<>();
                Float propane=0F;
                //氨气
                List<Object> ammoniaConsume=new ArrayList<>();
                Float ammonia=0F;
                int i=0;
                for (ParGasValue parGasValue:parGasValueList
                     ) {
                    //累计值的量首位置为0
                    if (i==0){
                        nit=parGasValue.getNitFlowValue();
                        met=parGasValue.getMethanolFlow();
                        propane=parGasValue.getPropaneFlow();
                        ammonia=parGasValue.getAmmoniaFlowValue();
                    }
                    i++;
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getNitFlowValue(),nit));
                    map.put("name","氮气");
                    map.put("unit","m³");
                    nitConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getMethanolFlow(),met));
                    map.put("name","甲醇");
                    map.put("unit","m³");
                    metConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getPropaneFlow(),propane));
                    map.put("name","丙烷");
                    map.put("unit","m³");
                    propaneConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getAmmoniaFlowValue(),ammonia));
                    map.put("name","氨气");
                    map.put("unit","m³");
                    ammoniaConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                gasList.add(nitConsume);
                gasList.add(metConsume);
                gasList.add(propaneConsume);
                gasList.add(ammoniaConsume);
                jsonObject.put("gas",gasList);
            }
            if (kilnType==3){
                List<Object> gasList=new ArrayList<>();
                //氮气
                List<Object> nitConsume=new ArrayList<>();
                Float nit=0F;
                //二氧化碳
                List<Object> carbonConsume=new ArrayList<>();
                Float carbon=0F;
                //氨气
                List<Object> ammoniaConsume=new ArrayList<>();
                Float ammonia=0F;
                int i=0;
                for (ParGasValue parGasValue:parGasValueList
                ) {
                    //累计值的量首位置为0
                    if (i==0){
                        nit=parGasValue.getNitFlowValue();
                        carbon=parGasValue.getCarDioxideFlowValue();
                        ammonia=parGasValue.getAmmoniaFlowValue();
                    }
                    i++;
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getNitFlowValue(),nit));
                    map.put("name","氮气");
                    map.put("unit","m³");
                    nitConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getCarDioxideFlowValue(),carbon));
                    map.put("name","二氧化碳");
                    map.put("unit","m³");
                    carbonConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parGasValue.getCreateTime());
                    map.put("value",floatSubtract(parGasValue.getAmmoniaFlowValue(),ammonia));
                    map.put("name","氨气");
                    map.put("unit","m³");
                    ammoniaConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                gasList.add(nitConsume);
                gasList.add(carbonConsume);
                gasList.add(ammoniaConsume);
                jsonObject.put("gas",gasList);
            }
        }
        //油搅拌转速趋势
        List<ParRotSpeedValue> parRotSpeedValueList = parRotSpeedValueServiceBiz.list(new QueryWrapper<ParRotSpeedValue>()
                .eq(ParEleValue.KILN_ID, kilnId)
                .eq(ParEleValue.TASK_ID, electricQueryParam.getTaskId()));
        if (parRotSpeedValueList.size()>0){
            if (kilnType==1){
                List<Object> rotList=new ArrayList<>();
                List<Object> rot1Consume=new ArrayList<>();
                List<Object> rot2Consume=new ArrayList<>();
                for (ParRotSpeedValue parRotSpeedValue:parRotSpeedValueList
                ) {
                    map.put("time",parRotSpeedValue.getCreateTime());
                    map.put("value",parRotSpeedValue.getOilStiSpeedAValue());
                    map.put("name","1号油搅拌转速");
                    map.put("unit","转/min");
                    rot1Consume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parRotSpeedValue.getCreateTime());
                    map.put("value",parRotSpeedValue.getOilStiSpeedBValue());
                    map.put("name","2号油搅拌转速");
                    map.put("unit","转/min");
                    rot2Consume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                rotList.add(rot1Consume);
                rotList.add(rot2Consume);
                jsonObject.put("rot",rotList);
            }
        }
        //温度趋势（实际温度、设定温度）
        List<ParTemValue> parTemValueList = parTemValueServiceBiz.list(new QueryWrapper<ParTemValue>()
                .eq(ParEleValue.KILN_ID, kilnId)
                .eq(ParEleValue.TASK_ID, electricQueryParam.getTaskId()));
        if (parTemValueList.size()>0){
            if (kilnType==1){
                List<Object> tempList=new ArrayList<>();
                List<Object> actualTempConsume=new ArrayList<>();
                List<Object> setupTempConsume=new ArrayList<>();
                List<Object> actualOilTempConsume=new ArrayList<>();
                List<Object> setupOilTempConsume=new ArrayList<>();
                for (ParTemValue parTemValue:parTemValueList
                     ) {
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getActTemValue());
                    map.put("name","实际温度");
                    map.put("unit","℃");
                    actualTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getSetTemValue());
                    map.put("name","设定温度");
                    map.put("unit","℃");
                    setupTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getOilTankActTemValue());
                    map.put("name","油槽实际温度");
                    map.put("unit","℃");
                    actualOilTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getOilTankSetTemValue());
                    map.put("name","油槽设定温度");
                    map.put("unit","℃");
                    setupOilTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                tempList.add(actualTempConsume);
                tempList.add(setupTempConsume);
                tempList.add(actualOilTempConsume);
                tempList.add(setupOilTempConsume);
                jsonObject.put("temp",tempList);
            }
            if (kilnType==2){
                List<Object> tempList=new ArrayList<>();
                List<Object> actualTempConsume=new ArrayList<>();
                List<Object> setupTempConsume=new ArrayList<>();
                for (ParTemValue parTemValue:parTemValueList
                ) {
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getActTemValue());
                    map.put("name","实际温度");
                    map.put("unit","℃");
                    actualTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getSetTemValue());
                    map.put("name","设定温度");
                    map.put("unit","℃");
                    setupTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                tempList.add(actualTempConsume);
                tempList.add(setupTempConsume);
                jsonObject.put("temp",tempList);
            }
            if (kilnType==3){

                List<Object> tempList=new ArrayList<>();
                List<Object> actualTempConsume=new ArrayList<>();
                List<Object> setupTempConsume=new ArrayList<>();
                List<Object> outerZone1TempConsume=new ArrayList<>();
                List<Object> outerZone2TempConsume=new ArrayList<>();
                for (ParTemValue parTemValue:parTemValueList
                ) {
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getActTemValue());
                    map.put("name","实际温度");
                    map.put("unit","℃");
                    actualTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getSetTemValue());
                    map.put("name","设定温度");
                    map.put("unit","℃");
                    setupTempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());
                    map.put("value",parTemValue.getOuterZone1Temp());
                    map.put("name","外一区温度");
                    map.put("unit","℃");
                    outerZone1TempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time",parTemValue.getCreateTime());

                    map.put("value",parTemValue.getOuterZone2Temp());
                    map.put("name","外二区温度");
                    map.put("unit","℃");
                    outerZone2TempConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                tempList.add(actualTempConsume);
                tempList.add(setupTempConsume);
                tempList.add(outerZone1TempConsume);
                tempList.add(outerZone2TempConsume);
                jsonObject.put("temp",tempList);
            }
        }
        return R.ok(jsonObject.toJSONString());
    }
    private Float floatSubtract(Float a,Float b){
        BigDecimal a1 = new BigDecimal(Float.toString(a));
        BigDecimal b1 = new BigDecimal(Float.toString(b));
        return a1.subtract(b1).floatValue();
    }

}
