package com.mt.wms.empty.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.HttpClient;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.controller.ApmsController;
import com.mt.wms.empty.enums.TaskTypeEnum;
import com.mt.wms.empty.params.AutoCurrTaskParam;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.service.*;
import com.mt.wms.empty.task.RunTaskUtils;
import com.mt.wms.empty.task.TaskDistanceUtils;
import com.mt.wms.empty.vo.ApmsEndProcessVo;
import com.mt.wms.empty.vo.CurrTaskMainQueryVo;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import com.mt.wms.empty.vo.CurrTaskVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class CurrTaskServiceImpl extends BaseService implements CurrTaskService {

    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    CurrTaskServiceBiz currTaskServiceBiz;

    @Autowired
    CurrTaskDetService currTaskDetService;

    @Autowired
    TaskHisService taskHisService;

    @Autowired
    TaskDetHisService taskDetHisService;

    @Autowired
    ApmsController apmsControl;

    @Autowired
    OrderInfoServiceBiz orderInfoServiceBiz;
    @Autowired
    CraftInfoServiceBiz craftInfoServiceBiz;
    @Autowired
    VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Autowired
    TaskDistanceUtils taskDistanceUtils;
    @Autowired
    RunTaskUtils runTaskUtils;
    @Autowired
    AutoExeTaskServiceBiz autoExeTaskServiceBiz;
    @Autowired
    KilnInfoServiceBiz kilnInfoServiceBiz;
    @Autowired
    LocationServiceBiz locationServiceBiz;
    @Autowired
    CurrTaskDetServiceBiz currTaskDetServiceBiz;

    @Override
    public CurrTaskVo getCurrTask(IdParam idParam) {
        CurrTask byId = currTaskServiceBiz.getById(idParam.getId());
        CurrTaskVo build = CurrTaskVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        return build;
    }

    @Override
    public CurrTask getCurrTaskById(Long id) {
        return currTaskServiceBiz.getById(id);
    }

    @Override
    public void updateCurrTaskById(CurrTask id) {
        currTaskServiceBiz.updateById(id);
    }

    @Override
    public R<IdVo> createProcessTask(TaskCreateParam param) {
        Assert.notNull("错误，加工类型不可为空！", param.getProcessType());
        CurrTask currTask = new CurrTask();
        BeanUtils.copyProperties(param, currTask);
        currTask.setTaskType(TaskTypeEnum.WTK.getValue());
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(currTask.getKilnId());
        if (param.getAutoTaskType()!=0){
            currTask.setIsAuto(1);
        }

        //根据工艺号查询工艺id以及工艺plc值
        CraftInfo one = craftInfoServiceBiz.getOne(new QueryWrapper<CraftInfo>().eq(CraftInfo.CRAFT_CODE, param.getCraftCode()));
        currTask.setTargetPosition(kilnInfo.getCode());
        currTask.setCraftCodeId(one.getId());
        currTask.setPlcValue(one.getPlcValue());
        currTask.setInterCode(IDGenerator.gen("RW", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "TASK_CODE"));
        currTask.setTaskCode(currTask.getInterCode());
        setCommonField(currTask);
        currTaskServiceBiz.save(currTask);
        return successful(IdVo.builder().id(currTask.getId()).build());
    }

    @Override
    public R<PageVo<CurrTaskQueryVo>> currentTaskPage(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        currTaskQueryWrapper.in(CurrTask.STATUS,0,1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        return successful(new PageVo<>(page, CurrTaskQueryVo.class));
    }

    @Override
    public R<String> completeTask(IdParam param) {
        CurrTask byId = currTaskServiceBiz.getById(param.getId());
        TaskHis taskHis = new TaskHis();
        BeanUtils.copyProperties(byId, taskHis);
        taskHis.setStatus(2);
        taskHis.setId(null);
        taskHis.setInterCode(IDGenerator.gen("RWHIS", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "TASKHIS_CODE"));
        setCommonField(taskHis);
        //将主任务放入到历史表，清空id，防止重复
        R<IdVo> save = taskHisService.save(taskHis);
        byId.setStatus(2);
        currTaskServiceBiz.removeById(byId);
        List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(byId.getId()).build());
        List<TaskDetHis> taskDetHis = com.mt.wms.core.utils.BeanUtils.copyList(taskDetailsByMainId, TaskDetHis.class);
        taskDetHis.forEach(detHis -> {
            detHis.setTaskId(save.getData().getId());
        });
        //将详细任务信息放入详细历史表
        taskDetHisService.saveList(taskDetHis);
        return successful("执行成功。");
    }

    @Override
    public R<String> deleteBySheetNo(String sheetNo) {
        //一个生产单可能对应多个currTask,删除的时候查询是否对应多个，如果对应多个，自动循环删除所有currTask,当删除最后一条currTask时上报apms
        List<CurrTask> currTaskList = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.SHEET_NO, sheetNo));
        if (currTaskList.size()>1){
            int i=0;
            for (CurrTask currTask:currTaskList
                 ) {
                i++;
                if(currTask.getStatus()==0){
                    if (i==currTaskList.size()){
                        R<ApmsEndProcessVo> apmsStartProcessR = apmsControl.deleteBySheetNo(sheetNo);
                        if (!apmsStartProcessR.getData().getSuccess()){
                            return failed(apmsStartProcessR.getData().getMsg());
                        }
                    }
                    Long currTaskId = currTask.getId();
                    if (currTask.getIsAuto()==1){
                        List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                        if (currTaskList2.size()==1){
                            autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                        }
                    }
                    currTaskServiceBiz.removeById(currTaskId);
                    currTaskDetService.deleteByCurrTaskId(currTaskId);

                }
                if (currTask.getStatus()==2){
                    return failed("无法删除，任务已加工完成。");
                }if (currTask.getStatus()==1){
                    return failed("无法删除，任务正在执行中。");
                }
            }
            return successful(0,"APMS生产单删除成功。","");
        }else {
            for (CurrTask currTask:currTaskList
                 ) {
                if (currTask.getStatus()==0){
                    R<ApmsEndProcessVo> apmsStartProcessR = apmsControl.deleteBySheetNo(sheetNo);
                    if (apmsStartProcessR.getData().getSuccess())
                    {
                        Long currTaskId = currTask.getId();
                        if (currTask.getIsAuto()==1){
                            List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                            if (currTaskList2.size()==1){
                                autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                            }
                        }
                        currTaskServiceBiz.removeById(currTaskId);
                        currTaskDetService.deleteByCurrTaskId(currTaskId);
                        if (currTask.getIsAuto()==1){
                            List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                            if (currTaskList2.size()==0){
                                autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                            }
                        }
                        return successful(0,"APMS生产单删除成功。","");
                    }
                    else
                    {
                        return failed(apmsStartProcessR.getData().getMsg());
                    }
                }
                if (currTask.getStatus()==2){
                    return failed("无法删除，任务已加工完成。");
                }if (currTask.getStatus()==1){
                    return failed("无法删除，任务正在执行中。");
                }

            }
        }
        return failed("根据生产单号删除任务失败！");

    }

    @Override
    public R<String> deleteById(IdParam idParam) {
        CurrTask currTask = currTaskServiceBiz.getById(idParam.getId());
        if (currTask.getIsAuto()==1){
            List<CurrTask> currTaskList = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
            for (CurrTask currTask1:currTaskList
                 ) {
                if (currTask1.getStatus()!=0){
                    return R.failed("该自动任务已有步骤在执行中或执行完成，不能删除！");
                }
            }
            //能删除
            autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
            for (CurrTask currTask1:currTaskList
                 ) {
                currTaskDetServiceBiz.remove(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID,currTask1.getId()));
                currTaskServiceBiz.removeById(currTask1);
            }
            return R.ok(idParam.getId().toString());
        }else {
            currTaskServiceBiz.removeById(idParam);
            return R.ok(idParam.getId().toString());
        }

    }

    @Override
    public R<PageVo<CurrTaskMainQueryVo>> currentTaskMainPage(BasePageParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(CurrTask.STATUS, 1);
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        PageVo<CurrTaskMainQueryVo> pageVo = new PageVo<>(page, CurrTaskMainQueryVo.class);
        Assert.notNull("无执行中的任务！", pageVo.getRecords());
        pageVo.getRecords().forEach(e ->
        {
            //根据当前主任务查详细任务
            List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(e.getId()).build());
            ArrayList<String> materialDes = new ArrayList<>();
            ArrayList<String> iden = new ArrayList<>();
            //写入材料名与标识卡号
            taskDetailsByMainId.forEach(a -> {
                materialDes.add(a.getMaterialDes());
                iden.add(a.getIdenCardNum());
            });
            e.setMaterials(materialDes);
            e.setIdens(iden);
            ArrayList<String> customers = new ArrayList<>();
            //根据标识卡号查订单的客户名并写入
            e.getIdens().forEach(id -> {
                        OrderInfo one = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, id));
                        customers.add(one.getCustomerName());
                    }
            );
            e.setCustomers(customers);
            e.setKilnName(e.getTargetPosition());
            //e.setLocationName(locationServiceBiz.getById(e.getLocationId()).getLocationName());
        });
        return successful(pageVo);
    }

    @Override
    public R runTask(Long taskId) throws InterruptedException, IOException {
        CurrTask currTask = currTaskServiceBiz.getById(taskId);
        //是否为自动任务
        if (currTask.getIsAuto()==1){
            List<AutoExeTask> exeTaskList = autoExeTaskServiceBiz.list(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.F_TASK_ID, taskId));
            if (exeTaskList.size()==0){
                return R.failed("该任务为多步骤加工任务且该步骤不是第一步，请选择第一步骤任务执行！");
            }
        }
        return runTaskUtils.runTaskForInKiln(taskId);
    }

    @Override
    public R<String> createAutoTask(AutoCurrTaskParam autoCurrTaskParam) {
        return null;
    }
}
