package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.OrderInfo;
import com.mt.wms.core.dal.entity.TaskDetHis;
import com.mt.wms.core.dal.service.OrderInfoServiceBiz;
import com.mt.wms.core.dal.service.TaskDetHisServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.CurrTaskDetQueryParam;
import com.mt.wms.empty.params.OrderInfoTaskDetParam;
import com.mt.wms.empty.service.TaskDetHisService;
import com.mt.wms.empty.service.TaskHisService;
import com.mt.wms.empty.vo.CurrTaskVo;
import com.mt.wms.empty.vo.OrderInfoTaskDetVo;
import com.mt.wms.empty.vo.TaskHisDetVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class TaskDetHisServiceImpl extends BaseService implements TaskDetHisService {

    @Autowired
    TaskDetHisServiceBiz taskDetHisServiceBiz;
    @Autowired
    TaskHisService taskhisService;
    @Autowired
    OrderInfoServiceBiz orderInfoService;

    @Override
    public R<String> save(TaskDetHis taskDetHis) {
        taskDetHisServiceBiz.save(taskDetHis);
        return successful("操作成功");
    }

    @Override
    public R<String> saveList(List<TaskDetHis> taskDetHis) {
        taskDetHisServiceBiz.saveBatch(taskDetHis);
        return successful("操作成功");
    }

    @Override
    public PageVo<OrderInfoTaskDetVo> getHisOrderHisTask(OrderInfoTaskDetParam param) {
        QueryWrapper<TaskDetHis> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(TaskDetHis.IDEN_CARD_NUM, param.getIdenCardNum())
                .eq(TaskDetHis.ORDER_NO, param.getOrderNo())
                .eq(TaskDetHis.VALID, 1);
        Page<TaskDetHis> page = taskDetHisServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), queryWrapper);
        return new PageVo<>(page, OrderInfoTaskDetVo.class);
    }

    @Override
    public HashMap<String, Double> getSumByIdCard(String idCard) {
        HashMap<String, Double> result = new HashMap<>(2);
        QueryWrapper<TaskDetHis> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(TaskDetHis.IDEN_CARD_NUM, idCard)
                .eq(TaskDetHis.VALID, 1);
        List<TaskDetHis> list = taskDetHisServiceBiz.list(queryWrapper);
        list.forEach(e -> {
            CurrTaskVo byId = taskhisService.getById(IdParam.builder().id(e.getTaskId()).build());
            //如果加工类型是复加工，或者任务作废。移除对象
            if (byId.getProcessType().equals(2) || byId.getValid() != 1)
            {
                list.remove(e);
            }
        });
        //计算加工总数
        double sumWeight = list.stream().mapToDouble(TaskDetHis::getWeight).sum();
        double sumQuantity = list.stream().mapToDouble(TaskDetHis::getQuantity).sum();
        result.put("weight", sumWeight);
        result.put("quantity", sumQuantity);
        return result;
    }

    @Override
    public R<PageVo<TaskHisDetVo>> getTaskDetailsPage(CurrTaskDetQueryParam currTaskId) {
        QueryWrapper<TaskDetHis> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(TaskDetHis.TASK_ID, currTaskId.getCurrTaskId())
                .eq(TaskDetHis.VALID, 1);
        Page<TaskDetHis> page = taskDetHisServiceBiz.page(new Page<>(currTaskId.getCurrent(), currTaskId.getSize()), queryWrapper);
        PageVo<TaskHisDetVo> taskHisDetVoPageVo = new PageVo<>(page, TaskHisDetVo.class);
        taskHisDetVoPageVo.getRecords().forEach(
                e -> {
                    OrderInfo one = orderInfoService.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, e.getIdenCardNum()));
                    e.setProductModel(one.getProductModel());
                    e.setCustomer(one.getCustomerName());
                }
        );
        return successful(taskHisDetVoPageVo);
    }
}
