package com.mt.wms.empty.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.dal.entity.PointInfo;
import com.mt.wms.core.dal.entity.VehicleInfo;
import com.mt.wms.core.dal.service.PointInfoServiceBiz;
import com.mt.wms.core.utils.HttpClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: liguanghao
 * @Date: 2021/12/1 16:56
 * @Version 1.0
 */
@Component
public class TaskDistanceUtils {

    private final static Logger logger = LoggerFactory.getLogger(TaskDistanceUtils.class);

    @Autowired
    private PointInfoServiceBiz pointInfoServiceBiz;
    /**
     * 计算各个空闲小车执行此任务的路径长度，传入任务起终点，返回一个小车ID
     * @param startPoint 起点
     * @param endPoint 终点
     * @return 路径最短的小车id
     */
    public  Long chooseVehicle(String startPoint,String endPoint) throws IOException {
        //车辆是否在线，若不在线，返回在线的那一辆，都不在线返回0
        if (!rgvOnline(1L)){

            //液压台4 和 1号回火炉只能调用车辆1
            if ("BMC1".equals(startPoint)||"BMC1".equals(endPoint)||"BMC2".equals(startPoint)||"BMC2".equals(endPoint)||"YYT004".equals(startPoint)||"YYT004".equals(endPoint)){
                return 0L;
            }
            if (!rgvOnline(2L)){
                return 0L;
            }
            return 2L;
        }
        if (!rgvOnline(2L)){

            //3-4号回火炉只能调用车辆2
            if ("BMC3".equals(startPoint)||"BMC3".equals(endPoint)||"BMC4".equals(startPoint)||"BMC4".equals(endPoint)){
                return 0L;
            }
            if (!rgvOnline(1L)){
                return 0L;
            }
            return 1L;
        }
        //3-4号回火炉只能调用车辆2
        if ("BMC3".equals(startPoint)||"BMC3".equals(endPoint)||"BMC4".equals(startPoint)||"BMC4".equals(endPoint)){
            return 2L;
        }
        //液压台4 和 1号回火炉只能调用车辆1
        if ("BMC1".equals(startPoint)||"BMC1".equals(endPoint)||"YYT004".equals(startPoint)||"YYT004".equals(endPoint)){
            return 1L;
        }
        Double startDistance = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, startPoint)).getDistance();
        //  2022/2/19  两辆小车都空闲，获取出当前位置
        Double rgv1CurrPosition = RGVCurrPosition(1L);
        Double rgv2CurrPosition = RGVCurrPosition(2L);
        //计算空闲小车当前位置到起点的距离
        Double vehicle1Distance = Math.abs(startDistance - rgv1CurrPosition);
        Double vehicle2Distance = Math.abs(startDistance - rgv2CurrPosition);
        return  (vehicle1Distance < vehicle2Distance) ? 1L : 2L;

    }
    /**
     * 计算执行任务的小车本次任务的剩余路径经过的点位集合
     * @param vehicle 小车id
     * @param startPoint 起点
     * @param endPoint 终点
     * @return 点位集合
     */
    public  List getPointList(Long vehicle,String startPoint,String endPoint){
        Integer start = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, startPoint)).getId();
        Integer end = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.CODE, endPoint)).getId();
        //获取点位list
        List<PointInfo> pointInfoList = pointInfoServiceBiz.list(new QueryWrapper<PointInfo>().orderByAsc(PointInfo.ID));
        // TODO: 2022/2/19  获取小车当前位置
        Integer nowPoint=3;
        //计算小车当前位置到任务起点-任务起点到任务终点的路径点集合
        //小车当前位置、起点、终点，找出最大最小值
        int min=(((nowPoint<start)?nowPoint:start)<end)?((nowPoint<start)?nowPoint:start):end;
        int max=(((nowPoint>start)?nowPoint:start)>end)?((nowPoint>start)?nowPoint:start):end;
        return pointInfoList.subList(min, max);
    }

    //判断小车是否冲突,传入选定小车的id,任务起终点,另一小车位置。
    public Boolean conflictForVehicle(Long vehicleId,String startPoint,String endPoint,Double otherNowPosition) throws IOException {
        if (!rgvOnline(1L)||!rgvOnline(2L)){
            return false;
        }
        Double startDistance = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, startPoint)).getDistance();
        Double endDistance = pointInfoServiceBiz.getOne(new QueryWrapper<PointInfo>().eq(PointInfo.NOTE, endPoint)).getDistance();
        //获取点位list
        List<PointInfo> pointInfoList = pointInfoServiceBiz.list(new QueryWrapper<PointInfo>().orderByAsc(PointInfo.ID));
        // 2022/2/19  获取小车当前位置
        Double rgvCurrPosition = RGVCurrPosition(vehicleId);
        //计算小车当前位置到任务起点-任务起点到任务终点的路径点集合
        //小车当前位置、起点、终点，找出最大最小值
        Double minDistance=(((rgvCurrPosition<startDistance)?rgvCurrPosition:startDistance)<endDistance)?((rgvCurrPosition<startDistance)?rgvCurrPosition:startDistance):endDistance;
        Double maxDistance=(((rgvCurrPosition>startDistance)?rgvCurrPosition:startDistance)>endDistance)?((rgvCurrPosition>startDistance)?rgvCurrPosition:startDistance):endDistance;
        //另一辆小车在在小车路径上比如冲突，不在小车路径上时判定一下安全距离（4米）
        if (otherNowPosition>=minDistance&&otherNowPosition<=maxDistance){
            return true;
        }else {
            if (Math.abs(otherNowPosition-maxDistance)<4){
                return true;
            }
            if (Math.abs(otherNowPosition-minDistance)<4){
                return true;
            }
            return false;
        }
    }
    /**
     * RGV1的当前位置
     * id 小车表示
     * @return 小车位置值（距离原点的值）
     */
    private Double RGVCurrPosition(Long id){
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        if (id==1){
            json.put("identifier","S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV1ToWCS.CurrentPosition");
        }
        if (id==2){
            json.put("identifier","S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV2ToWCS.CurrentPosition");
        }
        String result = null;
        try {
            result = HttpClient.httpPost("http://192.168.6.51:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("获取RGV的当前位置失败");
        }
        Double value = Double.valueOf(JSONObject.parseObject(result).get("result").toString());
        return value*0.0001;
    }

    /**
     * 车辆是否在线
     * @param vehicleId 车辆id
     * @return true/false
     * @throws IOException  plc访问失败
     */
    private Boolean rgvOnline(Long vehicleId) throws IOException {
        String identifier=null;
        if (vehicleId==1){
            identifier="S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV1ToWCS.Online";
        }
        if (vehicleId==2){
            identifier="S7-1200 station_2.Boutique.DB500_PLCToWCS.RGV2ToWCS.Online";
        }
        Map<String, Object> json = new HashMap();
        json.put("nameSpace", 6);
        json.put("plcName", "plc1");
        json.put("identifier",identifier);
        String result = null;
        try {
            result = HttpClient.httpPost("http://192.168.6.51:8009/opcua/read", JSON.toJSONString(json));
        } catch (IOException e) {
            e.printStackTrace();
            logger.info("获取车辆"+vehicleId+"当前是否在线失败");
        }
        Boolean online = Boolean.valueOf(JSONObject.parseObject(result).get("result").toString());
        return online;
    }
}
